

Ship = Core.class(Sprite)

local function clamp(x, min, max) return x < min and min or (x > max and max or x) end


function Ship:init()
	local _tex = Bitmap.new(Texture.new("ship.png"))
	self:addChild(_tex)
	self:setAnchorPoint(0.5,0.5)
	self.speed = 30
	self.rotSpeed = 400
	self.maxThrust = 200
	self.thrustIncrement =5
	self.thrust = 0
	self.drag = 0.97
	self.vx = 0
	self.vy = 0
	
	self.isRotatingLeft = false
	self.isRotatingRight = false
	self.isAccelerating = false
	
	self:addEventListener(Event.KEY_DOWN, self.onKeyDown, self)
	self:addEventListener(Event.KEY_UP, self.onKeyUp, self)

end

function Ship:update(dt)
	
	local x, y = self:getPosition()
	local rot = self:getRotation()
	local vx, vy = self.vx, self.vy
	local nx, ny, nrot
	
	if self.isRotatingLeft then
		nrot = rot - self.rotSpeed * dt
		self:setRotation(nrot)
	elseif self.isRotatingRight then
		nrot = rot + self.rotSpeed * dt
		self:setRotation(nrot)
	else
		nrot = rot
	end
	
	if self.isAccelerating then
		self.thrust = clamp(self.thrust+self.thrustIncrement, 0, self.maxThrust)
		local nThrust = self.thrust
	else 
		self.thrust = self.thrust * self.drag
	end
	
	nrot = math.rad(nrot)

	vx = vx + math.cos(nrot)*self.thrust * dt
	vy = vy + math.sin(nrot)*self.thrust * dt 
		
	nx, ny = x + vx , y + vy 
	nx, ny = clamp(nx, 0,1000), clamp(ny, 0, 1000)
	self:setPosition(nx, ny)
	
end

function Ship:onKeyDown(ev)
	if ev.keyCode == KeyCode.UP then
		self.isAccelerating = true
	end
	
	if ev.keyCode == KeyCode.LEFT then
		self.isRotatingLeft = true
	elseif ev.keyCode == KeyCode.RIGHT then
		self.isRotatingRight = true
	end
end

function Ship:onKeyUp(ev)
	if ev.keyCode == KeyCode.UP  then
		self.isAccelerating = false
	end
	if ev.keyCode == KeyCode.LEFT then
		self.isRotatingLeft = false
	elseif  ev.keyCode == KeyCode.RIGHT then
		self.isRotatingRight = false
	end
end
